@@ -115,6 +115,7 @@ import Control.Monad.Catch
     , SomeException
     )
 import Control.Monad (MonadPlus, liftM)
+import Control.Monad.Fail (MonadFail)
 import Control.Monad.Fix (MonadFix)
 import Control.Monad.IO.Class (MonadIO(liftIO))
 import Control.Monad.Trans.Control (MonadBaseControl(..))
@@ -171,7 +172,9 @@ liftMask maskVariant k = do
         unmask (Respond b  fb') = Respond b  (unmask . fb')
         unmask (M m)            = M $ do
             -- retrieve base's unmask and apply to merged action
-            Masked unmaskVariant <- liftIO $ readIORef ioref
+            unmaskVariant <- liftIO $ do
+                Masked unmaskVariant <- readIORef ioref
+                return unmaskVariant
             unmaskVariant (m >>= chunk >>= return . unmask)
         unmask (Pure q)         = Pure q
 
@@ -216,9 +219,28 @@ data Finalizers m = Finalizers
     in the event of exceptions.
 -}
 newtype SafeT m r = SafeT { unSafeT :: R.ReaderT (IORef (Maybe (Finalizers m))) m r }
-    deriving (Functor, Applicative, Alternative, Monad, MonadPlus, MonadFix,
-              EC.MonadError e, SC.MonadState s, WC.MonadWriter w, CC.MonadCont,
-              MonadThrow, MonadCatch, MonadMask, MonadIO, B.MonadBase b)
+    deriving
+    ( Functor
+    , Applicative
+    , Alternative
+    , Monad
+-- The derived instance for `MonadFail` requires a `MonadFail` instance for
+-- `ReaderT` which is first available in `transformers-0.5.0.0`
+#if MIN_VERSION_transformers(0,5,0)
+    , MonadFail
+#endif
+    , MonadPlus
+    , MonadFix
+    , EC.MonadError e
+    , SC.MonadState s
+    , WC.MonadWriter w
+    , CC.MonadCont
+    , MonadThrow
+    , MonadCatch
+    , MonadMask
+    , MonadIO
+    , B.MonadBase b
+    )
 
 instance MonadTrans SafeT where
     lift m = SafeT (lift m)
@@ -331,7 +353,7 @@ instance (MonadIO m, MonadCatch m, MonadMask m) => MonadSafe (SafeT m) where
                 Just (Finalizers n fs) ->
                     (Just $! Finalizers n (M.delete (unlock key) fs), ())
 
-instance (MonadSafe m) => MonadSafe (Proxy a' a b' b m) where
+instance MonadSafe m => MonadSafe (Proxy a' a b' b m) where
     type Base (Proxy a' a b' b m) = Base m
     liftBase = lift . liftBase
     register = lift . register
