@@ -34,13 +34,13 @@ withFile file ioMode = bracket (liftIO $ IO.openFile file ioMode) (liftIO . IO.h
 {-| Read lines from a file, automatically opening and closing the file as
     necessary
 -}
-readFile :: (MonadSafe m) => FilePath -> Producer' String m ()
+readFile :: MonadSafe m => FilePath -> Producer' String m ()
 readFile file = withFile file IO.ReadMode P.fromHandle
 {-# INLINABLE readFile #-}
 
 {-| Write lines to a file, automatically opening and closing the file as
     necessary
 -}
-writeFile :: (MonadSafe m) => FilePath -> Consumer' String m r
+writeFile :: MonadSafe m => FilePath -> Consumer' String m r
 writeFile file = withFile file IO.WriteMode P.toHandle
 {-# INLINABLE writeFile #-}
